{m25pxx serial eeprom access stuff }
const

ComLength    = 8;
AddLength    = 24;
DataLength   = 8;

{ Instructions }
WriteEnaCom   = $06;
WriteDisCom  = $04;
ReadStatCom  = $05;
WriteStatCom = $01;
ReadCom      = $03;
PageProgCom  = $02;
SectorEraCom = $D8;
BulkEraCom   = $C7;
PowerDown    = $B9;
ReadIDCom    = $AB;

Data_ComMask     = $80;
ADDMask      = $800000;

{ status Reg Bits }
WIP = $01;
WEL = $02;
BP0 = $04;
BP1 = $08;
SRWD = $80;

OneMegID = $10;
TwoMegID = $11;
FourMegID = $12;
EightMegID = $13;

type

PageBufferType = array[0..255] of byte;
PageBufferPointer = ^PageBufferType;
{
procedure SetCSHigh;
begin
end;

procedure SetCSLow;
begin
end;

procedure SetDinHigh;
begin
end;

procedure SetDinLow;
begin
end;

procedure SetClockHigh;
begin
end;

procedure SetClockLow;
begin
end;

function DataHighQ : boolean;
begin
end;
}

procedure LDisableInterrupts;
begin
  inline($FA);                                 { CLI }
end;

procedure LEnableInterrupts;
begin
  inline($FB);                                 { STI }
end;

procedure PutByte(data: byte);
var cmask,bit : byte;
begin
  cmask := Data_ComMask;
  for bit := 1 to ComLength do
  begin
    if (cmask and data) = 0 then SetDinLow else SetDinHigh;
    cmask := cmask shr 1;
    SetClockHigh;
    SetClockLow;
  end;
end;

function GetByte : byte;
var cmask,bit,tdata : byte;
begin
  cmask := Data_ComMask;
  tdata := 0;
  for bit := 1 to DataLength do
  begin
    if DataHighQ then tdata := tdata or cmask;
    cmask := cmask shr 1;
    SetClockHigh;
    SetClockLow;
  end;
  Getbyte := tdata;
end;

procedure PutAddress(add : longint);
var
amask : longint;
bit : byte;
begin
  amask := ADDMask;
  for bit := 1 to ADDLength do
  begin
    if (amask and add) = 0 then SetDinLow else SetDinHigh;
    amask := amask shr 1;
    SetClockHigh;
    SetClockLow;
  end;
end;

procedure Prefix;
begin
  SetCSHigh;
  SetDinLow;
  SetClockLow;
  SetCSLow;
end;

procedure Suffix;
begin
  SetCSHigh;
  SetDinLow;
end;

function ReadByte25(add: longint) : byte;
begin
  Prefix;
  PutByte(ReadCom);
  PutAddress(add);
  ReadByte25 := GetByte;
  Suffix;
end;

function ReadStatus25 : byte;
begin
  Prefix;
  PutByte(ReadStatCom);
  ReadStatus25 := GetByte;
  Suffix;
end;

function ReadID25: byte;
begin
  Prefix;
  PutByte(ReadIDCom);
  PutAddress(0);
  ReadID25 := GetByte;
  Suffix;
end;

procedure WriteEnable;
begin
  Prefix;
  PutByte(WriteEnaCom);
  Suffix;
end;

procedure WaitForWrite;
begin
  while (ReadStatus25 and WIP) <> 0 do;
end;

procedure WriteByte(add : longint; data : byte);
begin
  WriteEnable;
  Prefix;
  PutByte(PageProgCom);
  PutAddress(add);
  PutByte(data);
  Suffix;
  WaitForWrite;
end;

procedure WritePage(add : longint; pageBuf : PageBufferPointer);
var
index : word;
begin
  WriteEnable;
  Prefix;
  PutByte(PageProgCom);
  PutAddress(add); { note that add 0..7 should be 0}
  for index := 0 to 255 do
  begin
    PutByte(PageBuf^[index]);
  end;
  Suffix;
  WaitForWrite;
end;

procedure EraseSector(add: longint);
begin
  WriteEnable;
  Prefix;
  PutByte(SectorEraCom);
  PutAddress(add);
  Suffix;
  WaitForWrite;
end;

procedure EraseChip;
begin
  WriteEnable;
  Prefix;
  PutByte(BulkEraCom);
  Suffix;
  WaitForWrite;
end;
