/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "types.h"
#include <stdio.h>

#define HM2_PCI_VENDORID_XIO2001   (0x104C)
#define HM2_PCI_DEVICEID_XIO2001   (0x8240)

#define HM2_PCI_VENDORID_MESAPCI   (0x2718)

#define HM2_PCI_DEVICEID_MESA6I25  (0x6125)
#define HM2_PCI_DEVICEID_MESA5I25  (0x5125)

#define SPI_CTRL_REG 0x70
#define SPI_DATA_REG 0x74

#define DAV_MASK 0x04

#define COOKIE_REG   0x100
#define COOKIE_5i25 0x55AACAFE

#define BOOT_ADDRESS     0x00000
#define BOOT_BLOCK_SIZE  64

#define FALLBACK_ADDRESS 0x10000
#define USER_ADDRESS     0x80000

#define PAGE_SIZE 256
#define SECTOR_SIZE 65536

//////////////////////////
// spi eeprom interface
#define CMD_PAGE_WRITE   0x02
#define CMD_READ         0x03
#define CMD_READ_STATUS  0x05
#define CMD_WRITE_ENABLE 0x06
#define CMD_READ_IDROM   0xAB
#define CMD_SECTOR_ERASE 0xD8


#define WRITE_IN_PROGRESS_MASK 0x01

#define ID_EEPROM_1M 0x10
#define ID_EEPROM_2M 0x11
#define ID_EEPROM_4M 0x12
#define ID_EEPROM_8M 0x13

#define SET_CS_HIGH upci_write_u32(region, SPI_CTRL_REG, 1)
#define SET_CS_LOW  upci_write_u32(region, SPI_CTRL_REG, 0)

#define PREFIX  SET_CS_LOW
#define SUFFIX  SET_CS_HIGH
//////////////////////////////

int m5i25_open_region(int _reg);
void m5i25_close_region(void);

int m5i25_check_boot(void);
void m5i25_write_boot(void);
void m5i25_verify_area(char *fname, __u32 start_address);
void m5i25_write_area(char *fname, __u32 start_address);
int m5i25_print_bitfile_header(FILE *fp);
int m5i25_check_cookie(void);
int m5i25_identify_EEPROM(void);

int detect_board(__u16 vendor_id, __u16 device_id);
