/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>

#ifdef __WIN32__
#else
  #include <termios.h>
  #include "upci.h"
#endif

#include "mesa5i25.h"

void usage(void) {
   printf("Syntax:\n");
   printf("  wpo address value\n");
   printf("Write value directly to PCI map area\n");
   printf("  address - hex register address in 0-0xFFFF.\n");
   printf("  value - hex value in 0-0xFFFFFFFF.\n");
}

int main(int argc, char *argv[])
{
    int region;
    __u32 val, addr;
    int board5i25, board6i25, bridge6i25;
        
    if (argc == 3) {
        if ((sscanf(argv[1], "%x", &addr) == 1) && (sscanf(argv[2], "%x", &val) == 1)) {
        } else {
            printf("Unknown command line options: %s %s\n", argv[1], argv[2]);
            return 0;
        }
    } else {
        usage();
        return 0;
    }

    upci_reset();
    if (upci_scan_bus() == -1)
        return -1;

    bridge6i25 = detect_board(HM2_PCI_VENDORID_XIO2001, HM2_PCI_DEVICEID_XIO2001);
    board6i25 = detect_board(HM2_PCI_VENDORID_MESAPCI, HM2_PCI_DEVICEID_MESA6I25);
    board5i25 = detect_board(HM2_PCI_VENDORID_MESAPCI, HM2_PCI_DEVICEID_MESA5I25);

    if ((bridge6i25 == -1) && (board5i25 == -1)) {
        printf("Board not found\n");
        return -1;
    }

    region = m5i25_open_region(0);
    if (region == -1)
        return -1;

    upci_write_u32(region, addr, val);

    atexit(m5i25_close_region);

    return 0;
}

