
#ifndef __EEPROM_H
#define __EEPROM_H

#ifdef __linux__
    #include <pci/pci.h>
#elif _WIN32
    #include "libpci/pci.h"
#endif
#include <stdio.h>

#include "devices.h"

#define ID_EEPROM_1M 0x10
#define ID_EEPROM_2M 0x11
#define ID_EEPROM_4M 0x12
#define ID_EEPROM_8M 0x13
#define ID_EEPROM_16M 0x14

#define PAGE_SIZE   256
#define SECTOR_SIZE 65536
#define BOOT_BLOCK_SIZE 64

#define BOOT_ADDRESS     0x000000
#define FALLBACK_ADDRESS 0x010000
#define USER_ADDRESS     (active_board->user_address)

typedef struct {
    dev_type type;
    dev_access_method method;
    struct pci_dev *dev;
    struct pci_dev *bridge;     // if there is XIO2001
    char ip_addr[16];
    const char *fpga_part_number;
    u8 flash_id;
    u32 user_address;
    int verbose;
} mesa_board_t;

u32 (*read_cookie)(void);
u8 (*read_idrom)(void);
int (*verify_area)(char *fname, u32 start_address);
int (*write_area)(mesa_board_t *board, char *fname, u32 start_address);
void prepare_boot_block(u8 flash_id);
u32 calc_user_space(u8 flash_id);
char *print_idrom_type(u8 idrom);
const char *hm2_get_general_function_name(u8 gtag);
void errmsg(const char *funct, const char *fmt, ...);
int print_bitfile_header(FILE *fp, char *part_name);
int open_device(mesa_board_t *board);
void close_device(mesa_board_t *board);

#endif
