
#ifndef __FLASH_SPI_H
#define __FLASH_SPI_H

#ifdef __linux__
    #include <pci/pci.h>
#elif _WIN32
    #include "libpci/pci.h"
#endif

#include "devices.h"

#define DAV_MASK 0x04

#define CMD_PAGE_WRITE   0x02
#define CMD_READ         0x03
#define CMD_READ_STATUS  0x05
#define CMD_WRITE_ENABLE 0x06
#define CMD_READ_IDROM   0xAB
#define CMD_SECTOR_ERASE 0xD8

#define WRITE_IN_PROGRESS_MASK 0x01

//////

#define XIO2001_SBAD_STAT_REG 0x00B2
#define XIO2001_GPIO_ADDR_REG 0x00B4
#define XIO2001_GPIO_DATA_REG 0x00B6

#define DATA_MASK 0x80
#define ADDR_MASK 0x800000

#define CMD_LEN  8
#define DATA_LEN 8
#define ADDR_LEN 24

void (*set_cs_high)(void);

u8 read_byte(u32 addr);
u8 read_idrom_spi(void);
void erase_sector(u32 addr);
void write_page(u32 addr, char *buff, int buff_i);
void done_programming(void);
void open_spi_access(dev_access_method method);
void close_spi_access(dev_access_method method);

#endif
