
#ifdef __linux__
    #include <pci/pci.h>
    #include <arpa/inet.h>
#elif _WIN32
    #include <windows.h>
    #include <winsock2.h>
#endif
#include <stdio.h>
#include <time.h>

#include "hostmot2.h"
#include "eeprom.h"
#include "lbp16.h"

#ifdef __linux__
    extern int sd;
    extern socklen_t len;
#elif _WIN32
    extern SOCKET sd;
    extern int len;
#endif
extern struct sockaddr_in server_addr, client_addr;

hm2_id_t id;
hm2_idrom_t idrom;
hm2_module_descriptor_t module_desc[HM2_MODULES_DESC_COUNT];

void run(void) {
    lbp16_cmd_addr packet;
    lbp16_cmd_addr_data32 reset_wd_packet;
    int i;
    u32 addr, wd_stat;
    struct timespec ts, ts_left;

    printf("FIRMWARE CONFIGURATION\n");
    LBP16_INIT_PACKET4(packet, CMD_READ_HOSTMOT2_ADDR32_INCR(sizeof(id)/4), 0x100);
    sendto (sd, (char*) &packet, sizeof(packet), 0, (struct sockaddr *) &server_addr, sizeof(server_addr));
    recvfrom (sd, (char*) &id, sizeof(id), 0, (struct sockaddr *) &client_addr, &len);
    printf("  cookie: %X\n", id.cookie);
    printf("  config name: %.8s\n", id.name);

    LBP16_INIT_PACKET4(packet, CMD_READ_HOSTMOT2_ADDR32_INCR(sizeof(idrom)/4), id.offset_to_idrom);
    sendto (sd, (char*) &packet, sizeof(packet), 0, (struct sockaddr *) &server_addr, sizeof(server_addr));
    recvfrom (sd, (char*) &idrom, sizeof(idrom), 0, (struct sockaddr *) &client_addr, &len);
    printf("  IDROM version: %u\n", idrom.idrom_type);
    printf("  board name: %.8s\n", idrom.board_name);
    printf("  FPGA size: %u\n", idrom.fpga_size);
    printf("  FPGA pins: %u\n", idrom.fpga_pins);
    printf("  IO ports: %u\n", idrom.io_ports);
    printf("  IO width: %u\n", idrom.io_width);
    printf("  MODULES\n");

    addr = id.offset_to_idrom + idrom.offset_to_modules;
    for (i = 0; i < HM2_MODULES_DESC_COUNT; i++) {
        LBP16_INIT_PACKET4(packet, CMD_READ_HOSTMOT2_ADDR32_INCR(sizeof(hm2_module_descriptor_t)/4), addr);
        sendto (sd, (char*) &packet, sizeof(packet), 0, (struct sockaddr *) &server_addr, sizeof(server_addr));
        recvfrom (sd, (char*) &module_desc[i], sizeof(hm2_module_descriptor_t), 0, (struct sockaddr *) &client_addr, &len);

        if (module_desc[i].gtag == 0) break;
        printf("    %s, version %d [instances=%d]\n", hm2_get_general_function_name(module_desc[i].gtag),
            module_desc[i].version, module_desc[i].instances);
        printf("    --  Base addr: 0x%04X\n", module_desc[i].base_address);
        printf("    --  Registers: %u\n", module_desc[i].num_registers);
        module_desc[i].instance_stride = module_desc[i].instance_stride >> 4;
        if (module_desc[i].instance_stride == 0) {
            module_desc[i].instance_stride = idrom.instance_stride_0;
        } else if (module_desc[i].instance_stride == 1) {
            module_desc[i].instance_stride = idrom.instance_stride_1;
        }
        printf("    --  Instance stride: %u\n", module_desc[i].instance_stride);
        addr += sizeof(hm2_module_descriptor_t);
    }

    LBP16_INIT_PACKET8(reset_wd_packet, CMD_WRITE_HOSTMOT2_ADDR32(1), HM2_WD_TIMER_ADDR, 0xFFFFFFFF);
    sendto (sd, (char*) &reset_wd_packet, sizeof(reset_wd_packet), 0, (struct sockaddr *) &server_addr, sizeof(server_addr));

    LBP16_INIT_PACKET8(reset_wd_packet, CMD_WRITE_HOSTMOT2_ADDR32(1), HM2_WD_STATUS_ADDR, 0);
    sendto (sd, (char*) &reset_wd_packet, sizeof(reset_wd_packet), 0, (struct sockaddr *) &server_addr, sizeof(server_addr));

    LBP16_INIT_PACKET8(reset_wd_packet, CMD_WRITE_HOSTMOT2_ADDR32(1), HM2_RESET_WD_ADDR, 0x5A000000);
    LBP16_INIT_PACKET4(packet, CMD_READ_HOSTMOT2_ADDR32(1), HM2_WD_TIMER_ADDR);
    
    ts.tv_sec = 0;
    ts.tv_nsec = 100*1000*1000;    // 100ms
    while (1) {
    LBP16_INIT_PACKET4(packet, CMD_READ_HOSTMOT2_ADDR32(1), HM2_WD_STATUS_ADDR);
        sendto (sd, (char*) &packet, sizeof(packet), 0, (struct sockaddr *) &server_addr, sizeof(server_addr));
        recvfrom (sd, (char*) &wd_stat, sizeof(wd_stat), 0, (struct sockaddr *) &client_addr, &len);
        printf("    Watchdog bite: %u\n", wd_stat);
    LBP16_INIT_PACKET4(packet, CMD_READ_HOSTMOT2_ADDR32(1), HM2_ENC_COUNTER_ADDR);
        sendto (sd, (char*) &packet, sizeof(packet), 0, (struct sockaddr *) &server_addr, sizeof(server_addr));
        recvfrom (sd, (char*) &addr, sizeof(addr), 0, (struct sockaddr *) &client_addr, &len);
        printf("    Encoder 0: %u\n", addr & 0xFFFF);
        nanosleep(&ts, &ts_left);
    }

}
