
#ifndef __HOSTMOT2_H
#define __HOSTMOT2_H

#define HM2_AREA_SIZE          0x10000
#define HM2_MODULES_DESC_COUNT 32

#define HM2_SPI_CTRL_REG 0x0070
#define HM2_SPI_DATA_REG 0x0074
#define HM2_COOKIE_REG   0x0100

#define HM2_ID_ADDR            0x0100

#define HM2_LED_ADDR           0x0200

#define HM2_IDROM_ADDR         0x0400
#define HM2_PIN_DESC_ADDR      0x0600
#define HM2_WRITE_ENABLE_ADDR  0x0800
#define HM2_IRQ_DIV_ADDR       0x0900
#define HM2_IRQ_STATUS_ADDR    0x0A00
#define HM2_CLEAR_IRQ_ADDR     0x0B00

#define HM2_WD_TIMER_ADDR      0x0C00
#define HM2_WD_STATUS_ADDR     0x0D00
#define HM2_RESET_WD_ADDR      0x0E00

#define HM2_DMA_ENABLE_ADDR    0x0F00

#define HM2_IO_PINS_ADDR       0x1000
#define HM2_IO_DDR_ADDR        0x1100
#define HM2_IO_ALT_SOURCE_ADDR 0x1200
#define HM2_IO_OPEN_DRAIN_ADDR 0x1300
#define HM2_IO_OUT_INVERT_ADDR 0x1400

#define HM2_STEP_RATE_ADDR      0x2000
#define HM2_STEP_ACCUM_ADDR     0x2100
#define HM2_STEP_MODE_ADDR      0x2200
#define HM2_STEP_DIRSETUP_ADDR  0x2300
#define HM2_STEP_DIRHOLD_ADDR   0x2400
#define HM2_STEP_STEPLEN_ADDR   0x2500
#define HM2_STEP_STEPSPACE_ADDR 0x2600
#define HM2_STEP_TBL_DATA_ADDR  0x2700
#define HM2_STEP_TBL_LEN_ADDR   0x2800
#define HM2_STEP_DDS_ADDR       0x2900

#define HM2_WAVE_RATE_ADDR      0x2A00
#define HM2_WAVE_PDM_RATE_ADDR  0x2B00
#define HM2_WAVE_TBL_LEN_ADDR   0x2C00
#define HM2_WAVE_TBL_PTR_ADDR   0x2D00
#define HM2_WAVE_TBL_DATA_ADDR  0x2E00

#define HM2_ENC_COUNTER_ADDR    0x3000
#define HM2_ENC_LATCH_CTRL_ADDR 0x3100
#define HM2_ENC_TSS_DIV_ADDR    0x3200
#define HM2_ENC_TSS_COUNT_ADDR  0x3300
#define HM2_ENC_FLTR_RATE_ADDR  0x3400

#define HM2_MUXENC_COUNTER_ADDR    0x3500
#define HM2_MUXENC_LATCH_CTRL_ADDR 0x3600
#define HM2_MUXENC_TSS_DIV_ADDR    0x3700
#define HM2_MUXENC_TSS_COUNT_ADDR  0x3800
#define HM2_MUXENC_FLTR_RATE_ADDR  0x3900

#define HM2_RESV_CMD_REG_ADDR    0x3A00
#define HM2_RESV_CMD_DATA_ADDR   0x3B00
#define HM2_RESV_CMD_STATUS_ADDR 0x3C00
#define HM2_RESV_VEL_ADDR        0x3D00
#define HM2_RESV_POS_ADDR        0x3E00

#define HM2_PWM_VAL_ADDR         0x4000
#define HM2_PWM_MODE_ADDR        0x4100
#define HM2_PWM_DDS_ADDR         0x4200
#define HM2_PDM_DDS_ADDR         0x4300
#define HM2_PWM_ENABLE_ADDR      0x4400

#define HM2_TPPWM_VAL_ADDR       0x4500
#define HM2_TPPWM_ENABLE_ADDR    0x4600
#define HM2_TPPWM_SETUP_ADDR     0x4700





#define HM2_COOKIE       0x55AACAFE

#define HM2_GTAG_WATCHDOG         (2)
#define HM2_GTAG_IOPORT           (3)
#define HM2_GTAG_ENCODER          (4)
#define HM2_GTAG_STEPGEN          (5)
#define HM2_GTAG_PWMGEN           (6)
#define HM2_GTAG_UART_TX          (9)
#define HM2_GTAG_UART_RX         (10)
#define HM2_GTAG_TRANSLATIONRAM  (11)
#define HM2_GTAG_MUXED_ENCODER   (12)
#define HM2_GTAG_MUXED_ENCODER_SEL (13)
#define HM2_GTAG_BSPI            (14)
#define HM2_GTAG_TPPWM           (19)
#define HM2_GTAG_LED            (128)
#define HM2_GTAG_RESOLVER       (192)
#define HM2_GTAG_SMARTSERIAL    (193)

typedef struct {
    u32 cookie;
    u8  name[8];
    u32 offset_to_idrom;
} hm2_id_t;

typedef struct {
    u32 idrom_type;
    u32 offset_to_modules;
    u32 offset_to_pin_desc;
    u8 board_name[8];  // ascii string, but not NULL terminated!
    u32 fpga_size;
    u32 fpga_pins;
    u32 io_ports;
    u32 io_width;
    u32 port_width;
    u32 clock_low;
    u32 clock_high;
    u32 instance_stride_0;
    u32 instance_stride_1;
    u32 register_stride_0;
    u32 register_stride_1;
} hm2_idrom_t;

typedef struct {
    u8 gtag;
    u8 version;
    u8 clock_tag;
    u8 instances;
    u16 base_address;

    u8 num_registers;
    u8 instance_stride;
    u32 multiple_registers;
} hm2_module_descriptor_t;

void run(void);

#endif
